summaryrefslogtreecommitdiffstats
path: root/src/flood-context.cpp
diff options
context:
space:
mode:
authorKrzysztof Kosi??ski <tweenk.pl@gmail.com>2011-07-22 02:09:27 +0000
committerKrzysztof KosiƄski <tweenk.pl@gmail.com>2011-07-22 02:09:27 +0000
commit328fad57dbfb65e3bd31062021d5cc3081e68515 (patch)
tree55b02cfb325a87d994fefb0e4ea88311812e9444 /src/flood-context.cpp
parentClean up some commented-out code (diff)
downloadinkscape-328fad57dbfb65e3bd31062021d5cc3081e68515.tar.gz
inkscape-328fad57dbfb65e3bd31062021d5cc3081e68515.zip
Replace direct use of Cairo contexts and surfaces in the rendering tree
with wrappers which keep some extra information about the surface, amd NRRect and NRRectL use with Geom::Rect and Geom::IntRect. Should simplify implementing filter primitive subregions. (bzr r10347.1.17)
Diffstat (limited to 'src/flood-context.cpp')
-rw-r--r--src/flood-context.cpp69
1 files changed, 34 insertions, 35 deletions
diff --git a/src/flood-context.cpp b/src/flood-context.cpp
index 90278ac95..d93e0284d 100644
--- a/src/flood-context.cpp
+++ b/src/flood-context.cpp
@@ -53,6 +53,7 @@
#include "display/nr-arena-image.h"
#include "display/canvas-arena.h"
#include "display/cairo-utils.h"
+#include "display/drawing-context.h"
#include <2geom/pathvector.h>
#include "sp-item.h"
#include "sp-root.h"
@@ -805,8 +806,8 @@ static void sp_flood_do_flood_fill(SPEventContext *event_context, GdkEvent *even
Geom::Point origin(screen.min()[Geom::X],
document->getHeight() - screen.height() - screen.min()[Geom::Y]);
- origin[Geom::X] = origin[Geom::X] + (screen.width() * ((1 - padding) / 2));
- origin[Geom::Y] = origin[Geom::Y] + (screen.height() * ((1 - padding) / 2));
+ origin[Geom::X] += (screen.width() * ((1 - padding) / 2));
+ origin[Geom::Y] += (screen.height() * ((1 - padding) / 2));
Geom::Scale scale(zoom_scale, zoom_scale);
Geom::Affine affine = scale * Geom::Translate(-origin * scale);
@@ -817,44 +818,42 @@ static void sp_flood_do_flood_fill(SPEventContext *event_context, GdkEvent *even
NRGC gc(NULL);
gc.transform.setIdentity();
+
+ Geom::IntRect final_bbox = Geom::IntRect::from_xywh(0, 0, width, height);
- NRRectL final_bbox;
- final_bbox.x0 = 0;
- final_bbox.y0 = 0; //row;
- final_bbox.x1 = width;
- final_bbox.y1 = height; //row + num_rows;
-
- nr_arena_item_invoke_update(root, &final_bbox, &gc, NR_ARENA_ITEM_STATE_ALL, NR_ARENA_ITEM_STATE_NONE);
+ nr_arena_item_invoke_update(root, final_bbox, &gc, NR_ARENA_ITEM_STATE_ALL, NR_ARENA_ITEM_STATE_NONE);
int stride = cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, width);
guchar *px = g_new(guchar, stride * height);
-
- cairo_surface_t *s = cairo_image_surface_create_for_data(
- px, CAIRO_FORMAT_ARGB32, width, height, stride);
- cairo_t *ct = cairo_create(s);
- // cairo_translate not necessary here - surface origin is at 0,0
-
- SPNamedView *nv = sp_desktop_namedview(desktop);
- guint32 bgcolor = nv->pagecolor;
- // bgcolor is 0xrrggbbaa, we need 0xaarrggbb
- guint32 dtc = (bgcolor >> 8) | (bgcolor << 24);
-
- ink_cairo_set_source_rgba32(ct, bgcolor);
- cairo_set_operator(ct, CAIRO_OPERATOR_SOURCE);
- cairo_paint(ct);
- cairo_set_operator(ct, CAIRO_OPERATOR_OVER);
-
- nr_arena_item_invoke_render(ct, root, &final_bbox, NULL, NR_ARENA_ITEM_RENDER_NO_CACHE );
-
- cairo_surface_flush(s);
- cairo_destroy(ct);
- cairo_surface_destroy(s);
-
- // Hide items
- SP_ITEM(document->getRoot())->invoke_hide(dkey);
+ guint32 bgcolor, dtc;
+
+ { // this block limits the lifetime of DrawingContext
+ cairo_surface_t *s = cairo_image_surface_create_for_data(
+ px, CAIRO_FORMAT_ARGB32, width, height, stride);
+ Inkscape::DrawingContext ct(s, Geom::Point(0,0));
+ // cairo_translate not necessary here - surface origin is at 0,0
+
+ SPNamedView *nv = sp_desktop_namedview(desktop);
+ bgcolor = nv->pagecolor;
+ // bgcolor is 0xrrggbbaa, we need 0xaarrggbb
+ dtc = (bgcolor >> 8) | (bgcolor << 24);
+
+ ct.setSource(bgcolor);
+ ct.setOperator(CAIRO_OPERATOR_SOURCE);
+ ct.paint();
+ ct.setOperator(CAIRO_OPERATOR_OVER);
+
+ nr_arena_item_invoke_render(ct, root, final_bbox, NR_ARENA_ITEM_RENDER_NO_CACHE );
+
+ cairo_surface_flush(s);
+ cairo_surface_destroy(s);
+
+ // Hide items
+ SP_ITEM(document->getRoot())->invoke_hide(dkey);
+
+ nr_object_unref((NRObject *) arena);
+ }
- nr_object_unref((NRObject *) arena);
-
guchar *trace_px = g_new(guchar, width * height);
memset(trace_px, 0x00, width * height);